Option Strict On

Imports System
Imports System.Data
Imports System.Collections
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.Design
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Reflection    ' PropertyInfo definition

Namespace rw

#If Not NET_2_0 = 1 Then

    ''' -----------------------------------------------------------------------------
    ''' Project	 : schedule
    ''' Class	 : rw.SimpleTemplatedControlDesigner
    '''
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' SimpleTemplatedControlDesigner adds design-time support (courtesy of Andy Smith)
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Class SimpleTemplatedControlDesigner
        Inherits TemplatedControlDesigner

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Override this if you want to be able to render the control when not all the templates have been defined.
        ''' </summary>
        ''' <param name="templateName"></param>
        ''' <returns></returns>
        ''' -----------------------------------------------------------------------------
        Protected Overridable Function CanRenderWithTemplateEmpty(ByVal templateName As String) As Boolean
            If (templateName = "ItemTemplate") Then Return False
            Return True
        End Function 'CanRenderWithTemplateEmpty

        Public Overrides ReadOnly Property AllowResize() As Boolean
            Get
                Return Not Me.TemplatesAreEmpty() OrElse Me.InTemplateMode
            End Get
        End Property

        Public Overrides Function GetDesignTimeHtml() As String
            If Not CanRenderWithCurrentTemplates() Then
                Return Me.GetEmptyDesignTimeHtml()
            End If
            Dim designTimeHtml As String = String.Empty
            Try
                CType(Me.Component, Control).DataBind()
                designTimeHtml = MyBase.GetDesignTimeHtml()
            Catch ex As Exception
                designTimeHtml = Me.GetErrorDesignTimeHtml(ex)
            End Try
            Return designTimeHtml
        End Function 'GetDesignTimeHtml

        Protected Overrides Function GetEmptyDesignTimeHtml() As String
            Return CreatePlaceHolderDesignTimeHtml(EditTemplateInstructions)
        End Function 'GetEmptyDesignTimeHtml

        Protected Overrides Function GetErrorDesignTimeHtml(ByVal e As Exception) As String
            Return Me.CreatePlaceHolderDesignTimeHtml("There was an error rendering the control.")
        End Function 'GetErrorDesignTimeHtml

        Private _templateEditingVerbs() As TemplateEditingVerb
        Private _templateNames() As String

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Create a read-only property that determines whether the control
        ''' is using templates at design time.
        ''' </summary>
        ''' <returns>True if Templates are empty</returns>
        ''' -----------------------------------------------------------------------------
        Protected ReadOnly Property TemplatesAreEmpty() As Boolean
            Get
                Dim templateName As String
                For Each templateName In Me.TemplateNames
                    If Not GetTemplate(templateName) Is Nothing Then
                        Return False
                    End If
                Next templateName
                Return True
            End Get
        End Property

        Public Function CanRenderWithCurrentTemplates() As Boolean
            Dim templateName As String
            For Each templateName In Me.TemplateNames
                If GetTemplate(templateName) Is Nothing Then
                    Return Me.CanRenderWithTemplateEmpty(templateName)
                End If
            Next templateName
            Return True
        End Function 'CanRenderWithCurrentTemplates

        Private Function TemplateIsEmpty(ByVal templateName As String) As Boolean
            Return GetTemplate(templateName) Is Nothing
        End Function 'TemplateIsEmpty

        Protected Overridable ReadOnly Property EditTemplateInstructions() As String
            Get
                Return "Right click to edit the templates for this control"
            End Get
        End Property

        Private ReadOnly Property TemplateNames() As String()
            Get
                If _templateNames Is Nothing Then
                    Dim names As New ArrayList
                    Dim componentType As Type = CType(Me.Component, Object).GetType()
                    Dim prop As PropertyInfo
                    For Each prop In componentType.GetProperties()
                        If (prop.PropertyType.IsAssignableFrom(GetType(ITemplate)) And prop.Name <> "DataSource") Then
                            names.Add(prop.Name)
                        End If
                    Next prop
                    If names.Count > 0 Then
                        _templateNames = New String(names.Count - 1) {}
                        names.CopyTo(_templateNames)
                    End If
                End If
                Return _templateNames
            End Get
        End Property

        Private Function GetTemplate(ByVal templateName As String) As ITemplate
            Dim componentType As Type = CType(Me.Component, Object).GetType()
            Dim pi As PropertyInfo = componentType.GetProperty(templateName)
            Dim value As ITemplate = CType(pi.GetValue(Me.Component, Nothing), ITemplate)
            Return value
        End Function 'GetTemplate

        Private Sub SetTemplate(ByVal templateName As String, ByVal template As ITemplate)
            Dim componentType As Type = CType(Me.Component, Object).GetType()
            Dim pi As PropertyInfo = componentType.GetProperty(templateName)
            pi.SetValue(Me.Component, template, Nothing)
        End Sub 'SetTemplate

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Override the GetCachedTemplateEditingVerbs method to check whether the verbs have been changed.
        ''' If they have, a new collection of verbs is created for use with this designer.
        ''' </summary>
        ''' <returns></returns>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Function GetCachedTemplateEditingVerbs() As TemplateEditingVerb()
            If _templateEditingVerbs Is Nothing Then
                DisposeTemplateEditingVerbs()
                _templateEditingVerbs = New TemplateEditingVerb(TemplateNames.Length - 1) {}
                Dim i As Integer
                For i = 0 To _templateEditingVerbs.Length - 1
                    _templateEditingVerbs(i) = New TemplateEditingVerb(TemplateNames(i), i, Me)
                Next i
            End If
            Return _templateEditingVerbs
        End Function 'GetCachedTemplateEditingVerbs

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Override the CreateTemplateEditingFrame method to provide a frame in which to manipulate
        ''' templates at design time.
        ''' </summary>
        ''' <param name="verb"></param>
        ''' <returns></returns>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Function CreateTemplateEditingFrame(ByVal verb As TemplateEditingVerb) As ITemplateEditingFrame
            Dim frame As ITemplateEditingFrame = Nothing
            If Not (_templateEditingVerbs Is Nothing) Then
                Dim i As Integer
                For i = 0 To _templateEditingVerbs.Length - 1
                    If _templateEditingVerbs(i) Is verb Then
                        Dim teService As ITemplateEditingService = CType(GetService(GetType(ITemplateEditingService)), ITemplateEditingService)
                        If Not (teService Is Nothing) Then
                            frame = teService.CreateFrame(Me, verb.Text, New String() {Me.TemplateNames(i)})
                        End If
                    End If
                Next i
            End If
            Return frame
        End Function 'CreateTemplateEditingFrame

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Override the GetTemplateContent method to return the content contained in the template.
        ''' </summary>
        ''' <param name="editingFrame"></param>
        ''' <param name="templateName"></param>
        ''' <param name="allowEditing"></param>
        ''' <returns>String</returns>
        ''' -----------------------------------------------------------------------------
        Public Overrides Function GetTemplateContent(ByVal editingFrame As ITemplateEditingFrame, _
                ByVal templateName As String, ByRef allowEditing As Boolean) As String
            Dim content As String = ""
            allowEditing = True
            If Not (_templateEditingVerbs Is Nothing) Then
                Dim i As Integer
                For i = 0 To _templateEditingVerbs.Length - 1
                    If _templateEditingVerbs(i).Text = editingFrame.Verb.Text Then
                        Dim editedTemplate As ITemplate = GetTemplate(templateName)
                        If Not (editedTemplate Is Nothing) Then
                            content = GetTextFromTemplate(editedTemplate)
                        End If
                    End If
                Next i
            End If
            Return content
        End Function 'GetTemplateContent

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Override the SetTemplateContent method to set the content contained in the template.
        ''' </summary>
        ''' <param name="editingFrame"></param>
        ''' <param name="templateName"></param>
        ''' <param name="templateContent"></param>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub SetTemplateContent(ByVal editingFrame As ITemplateEditingFrame, _
                ByVal templateName As String, ByVal templateContent As String)
            If Not (_templateEditingVerbs Is Nothing) Then
                Dim i As Integer
                For i = 0 To _templateEditingVerbs.Length - 1
                    If _templateEditingVerbs(i).Text = editingFrame.Verb.Text Then
                        Dim newTemplate As ITemplate = Nothing
                        If Not (templateContent Is Nothing) AndAlso templateContent.Length > 0 Then
                            newTemplate = GetTemplateFromText(templateContent)
                        End If
                        SetTemplate(templateName, newTemplate)
                    End If
                Next i
            End If
        End Sub 'SetTemplateContent

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Create a method that releases resources used by the
        ''' designer verbs associated with design-time templates.
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Private Sub DisposeTemplateEditingVerbs()
            If Not (_templateEditingVerbs Is Nothing) Then
                Dim i As Integer
                For i = 0 To _templateEditingVerbs.Length - 1
                    _templateEditingVerbs(i).Dispose()
                Next i
                _templateEditingVerbs = Nothing
            End If
        End Sub 'DisposeTemplateEditingVerbs

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Override the Dispose method to release resources associated with this designer.
        ''' </summary>
        ''' <param name="disposing"></param>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
                DisposeTemplateEditingVerbs()
            End If
            MyBase.Dispose(disposing)
        End Sub 'Dispose

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Override the OnComponentChanged method to change
        ''' the control's DataSource or styles in the designer.
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="ce"></param>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub OnComponentChanged(ByVal sender As Object, ByVal ce As ComponentChangedEventArgs)
            MyBase.OnComponentChanged(sender, ce)
            If Not (ce.Member Is Nothing) Then
                Dim name As String = ce.Member.Name
                If name = "Font" OrElse name = "ForeColor" OrElse name = "BackColor" Then
                    OnStylesChanged()
                End If
            End If
        End Sub 'OnComponentChanged

        Protected Friend Sub OnStylesChanged()
            DisposeTemplateEditingVerbs()
        End Sub 'OnStylesChanged

    End Class 'SimpleTemplatedControlDesigner

#End If

End Namespace

